#ifndef GSKYDIR_HPP
#define GSKYDIR_HPP

/* __ Includes ___________________________________________________________ */
struct cache {
    double sin_dec;
    double cos_dec;
};

/* __ GSkyDir class ______________________________________________________ */
class GSkyDir {
public:
    GSkyDir(void) : m_ra(0.0), m_dec(0.0) {}
    GSkyDir(double ra, double dec) : m_ra(ra), m_dec(dec) {}
    virtual ~GSkyDir(void) {}
    double dist(const GSkyDir& dir) const;
protected:
    double m_ra;
    double m_dec;
};

/* __ GSkyDir2 class _____________________________________________________ */
class GSkyDir2 {
public:
    GSkyDir2(void) : m_ra(0.0), m_dec(0.0), m_has_cache(false) {}
    GSkyDir2(double ra, double dec) : m_ra(ra), m_dec(dec), m_has_cache(false) {}
    virtual ~GSkyDir2(void) {}
    double dist(const GSkyDir2& dir) const;
    const double& sin_dec(void) const;
    const double& cos_dec(void) const;
protected:
    double m_ra;
    double m_dec;
    mutable bool   m_has_cache;
    mutable double m_sin_dec;
    mutable double m_cos_dec;
};
inline const double& GSkyDir2::sin_dec(void) const
{
    if (!m_has_cache) {
        m_has_cache = true;
        m_sin_dec   = std::sin(m_dec);
        m_cos_dec   = std::cos(m_dec);
    }
    return m_sin_dec;
}
inline const double& GSkyDir2::cos_dec(void) const
{
    if (!m_has_cache) {
        m_has_cache = true;
        m_sin_dec   = std::sin(m_dec);
        m_cos_dec   = std::cos(m_dec);
    }
    return m_cos_dec;
}

/* __ GSkyDir3 class _____________________________________________________ */
class GSkyDir3 {
public:
    GSkyDir3(void) : m_ra(0.0), m_dec(0.0), m_has_cache(false) {}
    GSkyDir3(double ra, double dec) : m_ra(ra), m_dec(dec), m_has_cache(false) {}
    virtual ~GSkyDir3(void) {}
    double dist(const GSkyDir3& dir) const;
protected:
    double m_ra;
    double m_dec;
    mutable bool   m_has_cache;
    mutable double m_sin_dec;
    mutable double m_cos_dec;
};


/* __ GSkyDir4 class _____________________________________________________ */
class GSkyDir4 {
public:
    GSkyDir4(void) : m_ra(0.0), m_dec(0.0), m_cache(0) {}
    GSkyDir4(double ra, double dec) : m_ra(ra), m_dec(dec), m_cache(0) {}
    virtual ~GSkyDir4(void) { if (m_cache != 0) delete m_cache; }
    double dist(const GSkyDir4& dir) const;
protected:
    double m_ra;
    double m_dec;
    mutable cache* m_cache;
};

#endif